from dataclasses import dataclass

import itertools
from dataclasses_json import dataclass_json, Undefined, CatchAll
from typing import List, Tuple

from regioneval.libpagexml2 import PageXMLPage, TextEquiv, PageRegionTypes


class Baseline:
    pass

class FormatOnePage:
    pass

@dataclass_json
@dataclass
class F1FCreator:
    name: str
    version: str

@dataclass_json(undefined=Undefined.INCLUDE)
@dataclass
class F1FMeta:
    gid: int
    data: CatchAll

@dataclass_json(undefined=Undefined.INCLUDE)
@dataclass
class F1FAtom:
    gid: int
    type: str
    data: CatchAll

@dataclass_json
@dataclass
class FormatOneFile:
    filename: str
    alternatives: List[str]
    image_size: Tuple[int,int]
    meta: List[F1FMeta]
    content: List[F1FAtom]




def pagexml_to_format1(page: PageXMLPage):
    f = FormatOneFile(page.image_filename, alternatives=[], image_size=(page.image_width, page.image_height), meta=[], content=[])
    tl_gids = dict()
    gids = itertools.count(1)
    f.meta.append(F1FMeta(gid=next(gids), data={"layout-method": "full"}))
    for src_tl in page.text_lines():
        if src_tl.baseline is not None:
            bg = next(gids)
            f.content.append(F1FAtom(gid=bg, type="baseline"))#, data={"points":src_tl.baseline.tolist()}))
            g = next(gids)
            f.content.append(F1FAtom(gid=g, type="textline", data={"from": bg})) #, "polygon": src_tl.polygon.tolist()}))
            tl_gids[src_tl.id] = g
        else:
            g = next(gids)
            f.content.append(F1FAtom(gid=g, type="textline"))#, data={"polygon": src_tl.polygon.tolist()}))
            tl_gids[src_tl.id] = g
        tl_gid = g
        # store the text
        if TextEquiv.TEXT_ID_PREDICTION in src_tl.text_equivs:
            pred_gid = next(gids)
            f.content.append(F1FAtom(gid=pred_gid, type="text", data={"src": "pred", "for": tl_gid, "unicode": src_tl.text_equivs[TextEquiv.TEXT_ID_PREDICTION].text}))
        else:
            pred_gid = None
        if TextEquiv.TEXT_ID_GROUND_TRUTH in src_tl.text_equivs:
            gt_gid = next(gids)
            if pred_gid:
                f.content.append(F1FAtom(gid=gt_gid, type="text", data={"src": "gt", "for": tl_gid, "from": pred_gid,
                                                                          "unicode": src_tl.text_equivs[
                                                                              TextEquiv.TEXT_ID_GROUND_TRUTH].text}))
            else:
                f.content.append(F1FAtom(gid=gt_gid, type="text", data={"src": "gt", "for": tl_gid,
                                                                          "unicode": src_tl.text_equivs[
                                                                              TextEquiv.TEXT_ID_GROUND_TRUTH].text}))
    for reg in page.regions:
        reg_gid = next(gids)
        froms = [tl_gids[x.id] for x in reg.text_lines]
        data = {"lines": froms, 'type': reg.region_type.value}
        if reg.is_text() and reg.region_text_type:
            data["subtype"] = reg.region_text_type.value
        #data["polygon"] = reg.polygon.tolist()

        f.content.append(F1FAtom(gid=reg_gid, type="region", data=data))

    return f







